# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# groupvte 
# Copyright (C) 2005 Michele Baldessari <michele@pupazzo.org>

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.

from types import DictType, ListType
import gtk, gtk.gdk, sys, os.path, StringIO, traceback, pango, logging
from vteview import *
from settings import Settings
        
def trace_exception(type, value, t):
    trace = StringIO.StringIO()
    traceback.print_exception(type, value, t, None, trace)
    return trace.getvalue()

class ExceptionDialog:
    def __init__(self, parent, traceback):
        dialog = gtk.Dialog("Unknown error", parent,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            (gtk.STOCK_QUIT, gtk.RESPONSE_CANCEL))
        dialog.set_resizable(False)
        dialog.set_border_width(12)
        dialog.vbox.set_spacing(6)
        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
        hbox = gtk.HBox()
        hbox.set_spacing(4)
        hbox.pack_start(image)

        description = gtk.Label("An unknown error occured. Please report the text below, " \
            "along with what you were doing that may have caused the error. Send the whole backtrace to: " \
            "<michele@pupazzo.org>")
        description.set_justify(gtk.JUSTIFY_FILL)
        description.set_line_wrap(True)
        hbox.pack_start(description)
        dialog.vbox.pack_start(hbox)

        textview = gtk.TextView()
        textview.get_buffer().set_text(traceback)
        textview.modify_font(pango.FontDescription("Monospace 8"))
        scrolledwindow = gtk.ScrolledWindow()
        scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledwindow.add(textview)
                                                                    
        scrolledwindow.set_size_request(-1, 120)
        dialog.vbox.pack_start(scrolledwindow)
        self.dialog = dialog

    def run(self):
        self.dialog.show_all()
        return self.dialog.run() == gtk.RESPONSE_OK


class MultiTerminal:
    ui = '''
    <ui>
      <menubar name="MenuBar">
        <menu action="Main">
          <menuitem action="Switch session"/>
          <menuitem action="Quit"/>
        </menu>
        <menu action="Edit">
          <menuitem action="Copy"/>
          <menuitem action="Paste"/>
        </menu>
        <menu action="Tabs">
          <menuitem action="New"/>
          <menuitem action="Reset"/>
        </menu>
        <placeholder name="SessionsHere">
        </placeholder>
        <menu action="Help">
          <menuitem action="About"/>
        </menu>
      </menubar>
    </ui>'''

	def modified_cb(self, terminal, l):
		if self.vteview.get_current_tab() != l:
            self.vteview.set_label_color(l, self.conf.get_active_color())
		else:
            self.vteview.set_label_color(l, self.conf.get_inactive_color())
		
	def switch_page_cb(self, page, page_num, tab_number):
        self.vteview.set_label_color(tab_number, self.conf.get_inactive_color())

	def child_exited_cb(self, terminal, t):
        if self.vteview.get_nr_tabs() > 1:
            self.vteview.remove_tab(t)
        elif self.vteview.get_nr_tabs() == 1:
            gtk.main_quit()

    def copy_clipboard_cb(self):
        return
    
    def paste_clipboard_cb(self):
        return

	def add_terminal(self, label, commands=None):
        vt = VteStruct(self.conf)
		vt.term.connect("child-exited", self.child_exited_cb, self.vteview.get_nr_tabs())
		vt.term.connect("cursor-moved", self.modified_cb, self.vteview.get_nr_tabs())
		vt.term.connect("key_press_event", self.on_terminal_key_event)
		vt.term.connect("button_press_event", self.on_button_press_event)
		vt.scrollbar.set_adjustment(vt.term.get_adjustment())
		vt.hbox.pack_start(vt.term, True, True)
		vt.hbox.pack_start(vt.scrollbar, False, False)
        vt.label.set_text(label + " - " + str(self.vteview.get_nr_tabs()))
		self.vteview.append_tab(vt, self.window.get_size())
        self.vteview.set_current_tab(0)
		self.window.show_all()
		if commands != None:
			for i in commands:
                if not i.endswith("\n"):
                    vt.term.feed_child(i + "\n")
                else:
                    vt.term.feed_child(i)
	
	def execute(self):
		s = self.entry.get_text()
        if self.conf.is_command_allowed(s):
            self.vteview.execute_in_all_tabs(s)
            self.entry.set_text("")
        else:
            self.entry.set_text("%s is not in the allowed_commands list" % (s))

	def on_key_press_event_enter_cb(self, widget, event):
		keyname = gtk.gdk.keyval_name(event.keyval)
		if (keyname == "Return"):
			self.execute()
		
	def run_clicked_cb(self, widget, data):
		self.execute()

	def new_terminal_tab_cb(self, widget):
		self.add_terminal("New")

    def on_terminal_key_event(self, widget, event):
        mods = []
        keyname = gtk.gdk.keyval_name(event.keyval)
        if event.state & gtk.gdk.CONTROL_MASK:
            mods.append('ctrl')
        if event.state & gtk.gdk.SHIFT_MASK:
            mods.append('shift')
        
        mod = '_'.join(mods)
        if mod:
            name = '%s_%s' % (mod, keyname)
        else:
            name = '%s' % keyname

        if name == "ctrl_shift_C":
            if widget.get_has_selection():
                widget.copy_clipboard()
            return True
        elif name == "ctrl_shift_V":
            widget.paste_clipboard()
            return True
        return False

    def copy(self, widget):
        n = self.vteview.get_current_tab()
        vt = self.vteview.get_tab_terminal(n)
        if vt.get_has_selection():
            vt.copy_clipboard()

    def paste(self, widget):
        n = self.vteview.get_current_tab()
        vt = self.vteview.get_tab_terminal(n)
        vt.paste_clipboard()

    def on_key_press_event(self, widget, event):
        keyname = gtk.gdk.keyval_name(event.keyval)
        if event.state & gtk.gdk.MOD1_MASK:
            # <ALT>+<0> give focus to main input box
            if keyname == '0':
                self.entry.grab_focus()

            if keyname in ['1','2','3','4','5','6','7','8','9']:
                key = int(keyname)
                if key <= self.vteview.get_nr_tabs():
                    self.vteview.set_current_tab(key - 1)
                    return True
        return False

    def on_button_press_event(self, widget, event):
        tabnr = self.vteview.get_term_index(widget)
        if tabnr != -1:
            self.vteview.set_label_color(tabnr, self.conf.get_inactive_color())

        return False

    def about(self, action):
        dialog = gtk.AboutDialog()
        dialog.set_version(self.conf.get_version())
        dialog.set_copyright("Michele Baldessari (c) 2005")
        my_license = """
        This file may be distributed and/or modified under the terms of
        the GNU General Public License version 2 as published by
        the Free Software Foundation.
        This file is distributed without any warranty; without even the implied
        warranty of merchantability or fitness for a particular purpose.
        See "LICENSE.GPL" in the source distribution for more information.
        """

        dialog.set_license(my_license)
        dialog.set_website("http://michele.pupazzo.org/groupvte")
        dialog.set_authors(["Michele Baldessari <michele@pupazzo.org>"])
        dialog.set_artists(["Jakub Steiner"])
        print self.conf.get_pixmapdir()
        fimage = os.path.join(self.conf.get_pixmapdir(), 'groupvte.png')
        try:
            image = gtk.gdk.pixbuf_new_from_file(fimage)
            dialog.set_logo(image)
        except:
            traceback.print_exc()
            pass

        dialog.run()
        dialog.destroy()
    
    def delete_event(self, widget, event, data=None):
        self.quit(event)
        return False

    def quit(self, action):
        tabcount = self.vteview.get_nr_tabs()
        if tabcount > 0:
            dialog = gtk.Dialog('Quit', self.window,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_QUIT, gtk.RESPONSE_ACCEPT))
            dialog.set_resizable(False)
            dialog.set_border_width(12)
            dialog.vbox.set_spacing(6)
            image = gtk.Image()
            image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
            description = gtk.Label('There are <b>%d</b> terminals open\n' \
                'Are you sure you want to quit?' % (tabcount))
            description.set_use_markup(True)
            description.set_justify(gtk.JUSTIFY_FILL)
            description.set_line_wrap(True)

            dialog.vbox.pack_start(image)
            dialog.vbox.pack_start(description)

            dialog.vbox.show_all()
            r = dialog.run()
            dialog.destroy()
            if r == gtk.RESPONSE_ACCEPT:
                gtk.main_quit()
        else:
            gtk.main_quit()

    def reset_tabs_cb(self, w):
        self.vteview.reset_all_terminals()

    def new_session(self, session):
        if not self.sessions.has_key(session):
            return

        items = self.sessions[session]
        for i in items:
            connect = self.conf.get_command(i)
            name    = self.conf.get_machine_name(i)
            self.add_terminal(name, connect)

    def open_new_session(self, newsessionname):
        if not self.sessions.has_key(newsessionname):
            return
        self.vteview = VteView(self.conf)
        self.new_session(newsessionname)
		self.vbox.pack_start(self.vteview.view, True, True, 0)
        self.vbox.reorder_child(self.vteview.view, 1)
        self.vteview.view.show_all()
        self.cur_session = newsessionname

    def close_current_session(self):
        self.vteview.remove_all_tabs(True)
        self.vteview.destroy_view()
        del self.vteview

    def switch_session_cb(self, widget):
        dialog = gtk.Dialog('Switch session', self.window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        dialog.set_resizable(False)
        dialog.set_border_width(12)
        dialog.vbox.set_spacing(6)
        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
        label = gtk.Label("Choose session:")

        combobox = gtk.combo_box_new_text()
        counter  = 0
        active   = 0
        for i in self.conf.get_session_list():
            combobox.append_text(i)
            if i == self.cur_session:
                active = counter
            counter = counter + 1

        combobox.set_active(active)
        dialog.vbox.pack_start(image)
        dialog.vbox.pack_start(label)
        dialog.vbox.pack_start(combobox)

        dialog.vbox.show_all()
        r = dialog.run()
        dialog.destroy()
        if r == gtk.RESPONSE_REJECT:
            return

        model = combobox.get_model()
        index = combobox.get_active()
        newsession = model[index][0]
        if newsession == self.cur_session:
            return

        logging.debug("Close current session")
        self.close_current_session()

        logging.debug("Open new session %s" % newsession)
        self.open_new_session(newsession)

    def init_vbox(self):
		self.entry = gtk.Entry()
        self.entry.connect('key_press_event', self.on_key_press_event_enter_cb)
        colormap = self.entry.get_colormap()
        colorname = self.conf.get_color()
        try:
            colour = colormap.alloc_color(colorname)
        except:
            colour = colormap.alloc_color(self.conf.get_entry_color())

        style = self.entry.get_style().copy()
        style.base[gtk.STATE_NORMAL] = colour
        self.entry.set_style(style)
 
        self.ui_manager = gtk.UIManager()
		accel_group = self.ui_manager.get_accel_group()
        self.window.add_accel_group(accel_group)
        actiongroup = gtk.ActionGroup('Groupvte')

        # Create actions
        actiongroup.add_actions(
            [('Quit',     gtk.STOCK_QUIT,     '_Quit',
                None, 'Quit the Program',     self.quit),
             ('Switch session', None          ,'_Switch session',
                None, 'Switch session',  self.switch_session_cb),
             ('Main',     None,               '_Main'),
             ('Edit',     None,               '_Edit'),
             ('Tabs',     None,               '_Tabs'),
             ('New',      gtk.STOCK_NEW,      '_New',
                 None, 'New terminal tab..',   self.new_terminal_tab_cb),
             ('Reset',    gtk.STOCK_REFRESH,  '_Reset',
                None, 'Reset all tabs..',     self.reset_tabs_cb),
             ('Sessions', None,               '_Sessions'),
             ('Empty',    None,               '_Empty'),
             ('Help',     None,               '_Help'),
             ('About',    None,               '_About',
                 None, 'About Gropuvte..',     self.about)])
        copy  = gtk.Action('Copy',  'Copy',  'Copy from clipboard',  gtk.STOCK_COPY)
        paste = gtk.Action('Paste', 'Paste', 'Paste from clipboard', gtk.STOCK_PASTE)
        copy.connect("activate",  self.copy)
        paste.connect("activate", self.paste)

        actiongroup.add_action_with_accel(copy,  "<Ctrl><Shift>c")
        actiongroup.add_action_with_accel(paste, "<Ctrl><Shift>v")

        actiongroup.get_action('Quit').set_property('short-label', '_Quit')
		self.vbox = gtk.VBox()
        self.ui_manager.insert_action_group(actiongroup, 0)
        self.ui_manager.add_ui_from_string(self.ui)
        self.menubar = self.ui_manager.get_widget('/MenuBar')
        helpmenu = self.ui_manager.get_widget('/MenuBar/Help')
        helpmenu.set_right_justified(True)

        self.vbox.pack_start(self.menubar, False, True, 0)
		self.vbox.pack_start(self.vteview.view, True, True, 0)
		self.hbox = gtk.HBox()
		b = gtk.Button("E_xecute")
		b.connect("clicked", self.run_clicked_cb, "")
		self.hbox.pack_start(self.entry, True, True)
		self.hbox.pack_start(b, False, False, 0)

		self.vbox.pack_start(self.hbox, False, False)
		self.window.add(self.vbox)
        self.window.add_events(gtk.gdk.BUTTON_PRESS_MASK)
		self.window.connect("delete_event", self.delete_event)
        self.window.connect("key_press_event", self.on_key_press_event)
        self.window.connect("button_press_event", self.on_button_press_event)
		self.entry.grab_focus()
		self.window.show_all()
            
    def __cb_exception(self, type, value, trace):
        if type == KeyboardInterrupt:
            return

        traceback = trace_exception(type, value, trace)
        sys.stderr.write(traceback)
        if ExceptionDialog(self.window, traceback).run():
            self.window.run()
        else:
            sys.exit(1)

	def __init__(self, conf, session):
        sys.excepthook = self.__cb_exception
        
		self.window      = gtk.Window()
        self.conf        = conf
        self.sessions    = self.conf.get_session_hash()
        self.machines    = self.conf.get_machine_hash()
        self.cur_session = session
        self.vteview     = VteView(self.conf)
        self.init_vbox()
        self.new_session(self.cur_session)
