# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# groupvte - vteview
# Copyright (C) 2005 Michele Baldessari <michele@pupazzo.org>

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.

import gtk, gtk.gdk, logging
from settings import Settings

try:
    from vte import *
except:
    logging.warning("vte python module not found..trying _vte")
    try:
        from _vte import *
        logging.info("Successfully imported vte")
    except:
        logging.error("No vte nor _vte found, exiting")
        sys.exit(-1)


def print_all(s, widget):
    p = widget.get_parent()
    logging.debug("%s %s (parent is %s)" % (s, widget, p))
    try:
        c = widget.get_children()
        for i in c:
            t = i.get_parent()
            logging.debug("  Child %s has parent %s" % (i, t))
    except AttributeError:
        pass

class VteStruct:
    def __init__(self, conf):
		# Terminal defaults.
		self.audible = conf.get_bell()
		self.background = None
		self.blink = 0
		self.emulation = "xterm"
		(f, s) = conf.get_font_and_size()
        self.font = "%s %s" % (f, s)
		self.scrollback = 1000
		self.transparent = 0
		self.visible = 0
        self.term = Terminal()

		if self.background:
			self.term.set_background_image(self.background)
		if self.transparent:
			self.term.set_background_transparent(True)

		self.term.set_cursor_blinks(self.blink)
		self.term.set_emulation(self.emulation)
		self.term.set_font_from_string(self.font)
		self.term.set_scrollback_lines(self.scrollback)
		self.term.set_audible_bell(self.audible)
		self.term.set_visible_bell(self.visible)
		self.scrollbar = gtk.VScrollbar()
		self.hbox      = gtk.HBox()
		self.child_pid = self.term.fork_command()
        self.label     = gtk.Label()
        self.vbox      = gtk.VBox()
        self.vbox.pack_start(self.label, False, False)
        self.vbox.pack_start(self.hbox, True, True)

class VteView:
    def __init__(self, conf, terminals=[]):
        self.conf = conf
        self.currentterminal = 0
        self.terminals       = []
        self.view = gtk.Table(1, 1, True)

        if len(terminals) > 0:
            for i in terminals:
                self.append_tab(i)

    def get_tab_terminal(self, tab):
        try:
            return self.terminals[tab].term
        except:
            return None

    def get_tab_label(self, tab):
        try:
            return self.terminals[tab].label
        except:
            return None

	def switch_page_cb(self, page, page_num, tab_number):
        self.set_label_color(tab_number, self.conf.get_inactive_color())
    
    def set_label_color(self, tab, newcolor, bold=False):
        l = self.get_tab_label(tab)
        if not l:
            return

        if bold:
            l.set_markup("<span foreground='%s'><b>%s</b></span>" % \
                (newcolor, l.get_text()))
        else:
            l.set_markup("<span foreground='%s'>%s</span>" % \
                (newcolor, l.get_text()))

    def remove_tab(self, tab, fullyremove=True):
        self.view.remove(self.terminals[tab].vbox)
        i = self.terminals[tab]
        i.term.unparent()
        i.label.unparent()

        #i.term.hide()
        #i.label.hide()
        #i.hbox.hide()
        #i.vbox.hide()
        if fullyremove:
            del self.terminals[tab]

    def remove_all_tabs(self, fullyremove):
        if fullyremove:
            while len(self.terminals) > 0:
                self.remove_tab(0, fullyremove)
        else:
            for i in range(len(self.terminals)):
                self.remove_tab(i, fullyremove)

    def get_all_terms(self):
        l = []
        for i in self.terminals:
            l.append(i.term)
        return l

    def append_tab(self, vtestruct, size=None, needappend=True):
        if needappend:
            self.terminals.append(vtestruct)

        totalterminals = self.get_nr_tabs() - 1 
        maxcols = int(self.conf.get_max_columns())
        if totalterminals < maxcols:
            self.view.resize(1, totalterminals + 1)
            self.view.attach(vtestruct.vbox, totalterminals, totalterminals + 1, 0, 1)
        else:
            t = totalterminals % maxcols
            d = totalterminals / maxcols
            self.view.resize(maxcols, d - 1)
            self.view.attach(vtestruct.vbox, t, t + 1, d, d + 1)
            
    def get_nr_tabs(self):
        return len(self.terminals)

    def execute_in_all_tabs(self, command):
        if len(command.strip()) < 1:
            return
        if not command.endswith("\n"):
            command = command + "\n"
        for i in self.terminals:
            i.term.feed_child(command)

    def reset_all_terminals(self):
        for i in range(len(self.terminals)):
            self.terminals[i].term.reset(True, True)
            self.terminals[i].term.feed_child("\n")

    def get_term_index(self, terminal):
        for i in range(len(self.terminals)):
            if terminal == self.terminals[i].term:
                return i
        return -1

    def get_current_tab(self):
        for i in range(len(self.terminals)):
            if self.terminals[i].term.is_focus():
                return i
        return -1

    def set_current_tab(self, tab):
        self.terminals[tab].term.grab_focus()
        self.set_label_color(tab, self.conf.get_inactive_color())

    def destroy_view(self):
        self.view.destroy()
        del self.view
